function [ allState ] = dalec_run(driverFileName, useObs, daType, params)
%DALEC_RUN run the dalec model and perform DA
%
% driverFileName is the name of a file continaing driving meteorological
% datafor the dalec model and observation to assimilate
%
% useObs (optional) is a string containing a list of observations, 
% seperated by whitespace, to assimilate. Valid options are: 
% gpp, nee, cf, cw, rt, ra, lf, lw, cl, cs, cr
%
% params is a valid dalec parameter structure generated by dalec_initParams


%obseravtions to assimilate
if nargin<2
    useObs='all';
end
useObs=strsplit(upper(useObs));


%obseravtions to assimilate
if nargin<3
    daType='enkf';
end
daType=strsplit(lower(daType));


%initialise parameters (p) if not passed to the function
if nargin <4
    p=dalec_initParams();
else
    p=params;
end

%initialise state ensemble (s)
s=dalec_initState(p);

%read drivers (d) and observations (o)
[d, o]=dalec_readDrivers(driverFileName);

%filter obs based on users request
o=dalec_filterObs(o, useObs, p);

%pre-allocate a big space to hold the entire output
nSteps=size(d,1);
allState=zeros(nSteps,p.ndims,p.nrens);

%run the dalec & EnKF over the driving data
for timeStep=1:nSteps,
    
    oldEns=s;
    
    %unpack drivers and observations here
    %to avoid doing it inside deeper loops
    dDay=dalec_unpackDrivers(d,timeStep);
    
    %propagate the ensemble forward one 
    %time step (i.e. one day)
    s=dalec_propagate(s,p,dDay);
    
    %add stochasitc forcing to the ensemble
    %of state vectors
    s=dalec_stochastic(s,p);    
    
    %do the EnKF analysis
    [oDay,obsOp,obsLut]=dalec_unpackObs(o,p,timeStep);
    if oDay~=false;
        %disp(obsOp)
        if strcmp(daType,'enkf')
          s=dalec_EnKF_analysis(s,obsOp,p,oDay,obsLut);
        else
          s=dalec_PF_analysis(s,obsOp,p,oDay,obsLut);
        end
    end
    
    
    allState(timeStep,:,:)=s;
    
end

end

